/* sensor_sweep_applet.c - main source file for sensor_sweep_applet. */

/* Includes */
#include <applet-widget.h>

/* Sensor includes */
#include <sensors/sensors.h>
#include <sensors/error.h>
#include <sensors/chips.h>
#include <linux/sensors.h>

/* Local Includes */
#include "sensorFunc.h"
#include "properties.h"
#include "structures.h"

/* Function Prototypes */
GList *setupSensors(void);
gint updateGUI(gpointer data);
gint updateSensors(gpointer data);
void aboutWindowCB(AppletWidget *caller, gpointer data);
void setupApplet(struct appletInfo *ai);
void destroy(GtkWidget *widget, gpointer data);
void print_hello(GtkWidget *widget, gpointer data);
void updateGUIWrapper(GtkWidget *widget, GdkEventAny* e, gpointer data);

#define VERSION "0.10.4"

/* Main */
int main(int argc, char **argv) {
	gint ret;
	struct appletInfo *ai;
	FILE *configFile;
	GtkWidget *okdialog;

	/* Initialize the applet */
	applet_widget_init("sensor_sweep_applet", VERSION, argc, argv, NULL, 0, NULL);

	/* Allocate the memory for the main struct */
	ai = g_malloc(sizeof(struct appletInfo));
	ai->displayPos = 0;

	/* Open and initialize the configuration file */
	configFile = fopen("/etc/sensors.conf", "ro");
	if(configFile == NULL) {
		okdialog = gnome_ok_dialog("Could not open the lm_sensors config file (/etc/sensors.conf)");
		gtk_signal_connect(GTK_OBJECT(okdialog), "destroy", GTK_SIGNAL_FUNC(destroy), NULL);
		applet_widget_gtk_main();
		exit(1);
	}

	ret = sensors_init(configFile);
	if(ret != 0) {
		/* Could not initialize the config */
		if(ret == SENSORS_ERR_PROC) {
			/* Modules not loaded */
			okdialog = gnome_ok_dialog("/proc/sys/dev/sensors/chips or /proc/bus/i2c unreadable.\nMake sure you have the lm_sensor modules loaded.");
			gtk_signal_connect(GTK_OBJECT(okdialog), "destroy", GTK_SIGNAL_FUNC(destroy), NULL);
			applet_widget_gtk_main();
			exit(1);
		} else {
			/* Some other error */
			okdialog = gnome_ok_dialog("Could not initialize the config file.\nMake sure the lm_sensors modules are loaded.\n");
			gtk_signal_connect(GTK_OBJECT(okdialog), "destroy", GTK_SIGNAL_FUNC(destroy), NULL);
			applet_widget_gtk_main();
			exit(1); /* Exit the applet */
		}
	}

	/* Initialize the Gnome config system */
        gnome_config_push_prefix("/sensor_sweep_applet/");

	/* Setup the applets variables */
	setupApplet(ai);

	/* Setup the sensor Glist */
	ai->sensorList = setupSensors();
	ai->applet = applet_widget_new("sensor_sweep_applet");

	/* Set the size of the applet */
	gtk_widget_set_usize(GTK_WIDGET(ai->applet), ai->width, ai->height);

	/* Setup the panel right click menus */
	applet_widget_register_stock_callback(APPLET_WIDGET(ai->applet), "Properties", GNOME_STOCK_MENU_PROP, _("Properties"), properties, ai);
	applet_widget_register_stock_callback(APPLET_WIDGET(ai->applet), "About", GNOME_STOCK_MENU_ABOUT, _("About"), aboutWindowCB, NULL);

	/* Create and setup the event box widget */
	ai->eventbox = gtk_event_box_new();
	applet_widget_add(APPLET_WIDGET(ai->applet), ai->eventbox);
	gtk_widget_set_events(ai->eventbox, GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect(GTK_OBJECT(ai->eventbox), "button_press_event", GTK_SIGNAL_FUNC(updateGUIWrapper), ai);

	/* Create and setup the frame widget */
	ai->frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(ai->frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(ai->eventbox), ai->frame);

	/* Create and setup the alignment widget */
	ai->align = gtk_alignment_new(1, 1, 1, 1);
	gtk_container_add(GTK_CONTAINER(ai->frame), ai->align);

	/* The Vbox widget */
	ai->vbox = gtk_vbox_new(TRUE, TRUE);
	gtk_container_add(GTK_CONTAINER(ai->align), ai->vbox);

	/* Create the labels and pack them */
	ai->topLabel = gtk_label_new("");
	ai->bottomLabel = gtk_label_new("");
	gtk_box_pack_start(GTK_BOX(ai->vbox), ai->topLabel, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(ai->vbox), ai->bottomLabel, TRUE, TRUE, 0);

	/* Show the widget's */
	gtk_widget_show(ai->topLabel);
	gtk_widget_show(ai->bottomLabel);
	gtk_widget_show(ai->vbox);
	gtk_widget_show(ai->align);
	gtk_widget_show(ai->frame);
	gtk_widget_show(ai->eventbox);
	gtk_widget_show(ai->applet);

	/* Update the sensors */
	updateSensors(ai);
	ai->sensorsTimeoutTag = gtk_timeout_add(ai->freq, updateSensors, ai);

	/* Start the GUI update function that update the clist */
	updateGUI(ai);
	ai->GUITimeoutTag = gtk_timeout_add(ai->freq, updateGUI, ai);

	/* Start the applet */
	applet_widget_gtk_main();
	
	return(0);
}

/*************/
/* Functions */
/*************/

void print_hello(GtkWidget *widget, gpointer data) {
	/* Little function used to verify that a signal handler is being called */
	g_print("print_hello()\n");
}

void setupApplet(struct appletInfo *ai) {
	ai->height = gnome_config_get_int("Global/height=44");
	ai->width = gnome_config_get_int("Global/width=100");
	ai->freq = gnome_config_get_int("Global/freq=10000");
}

GList *setupSensors(void) {
	/* Read in the configured sensors and setup the GList structure to hold sensors */
	struct sensor *tempSensor;
	GList *list=NULL; /* Empty list */
	gint numSensors, i;
	gchar *string;

	/* Setup our string buffer */
	string = g_malloc(40);

	/* How many sensors are defined? Default to 0 */
	numSensors = gnome_config_get_int("Global/Number_Of_Sensors=0");

	if(numSensors != 0) {
		/* Iterate through sensors getting info */

		for(i=0;i<numSensors;i++) {
			tempSensor = g_malloc(sizeof(struct sensor));

			/* Read in the values */
			sprintf(string, "Sensor%i/chipName=NULL", i);
			tempSensor->chipName = gnome_config_get_string(string);
			sprintf(string, "Sensor%i/sensorName=NULL", i);
			tempSensor->sensorName = gnome_config_get_string(string);
			sprintf(string, "Sensor%i/displayName=NULL", i);
			tempSensor->displayName = gnome_config_get_string(string);

			/* If any of the above returned NULL there is a problem with that entry. Do not process */
			if(strcmp(tempSensor->chipName, "NULL") != 0 && strcmp(tempSensor->sensorName, "NULL") != 0 && strcmp(tempSensor->displayName, "NULL") != 0) {

				/* Get the sensor_chip_name struct filled out */
				if(getName(tempSensor->chipName, &tempSensor->nameStruct) != 0) {
					g_print("Could not get sensor Name\n");
					g_free(tempSensor);
				} else {
					/* Get the sensor number */
					tempSensor->sensorNum = getSensorNumber(&tempSensor->nameStruct, tempSensor->sensorName);
					if(tempSensor->sensorNum != 0) {
						/* Add it to the GList */
						list = g_list_append(list, tempSensor);
					} else {
						g_print("Could not get sensor number %i\n", i);
						g_free(tempSensor);
					}
				}
			} else {
				/* We don't need this GList entry since something is wrong with it */
				g_free(tempSensor);
			}
		}
	}

	/* Free the buffer */
	g_free(string);

	return(list);
}

void updateGUIWrapper(GtkWidget *widget, GdkEventAny* e, gpointer data) {
	/* GTK signal func wrapper to update the display */
	updateGUI(data);
}

gint updateGUI(gpointer data) {
	struct sensor *temp;
	gchar *string;
	struct appletInfo *ai;
	GList *list;
	gint pos;

	ai = (struct appletInfo *) data;

	list = ai->sensorList;

	/* Allocate our buffer */
	string = g_malloc(60);

	/* If this list is empty then just clear the UI labels and
	return FALSE so this timeout doesn't keep going */
	if(ai->sensorList == NULL) {
		gtk_label_set_text(GTK_LABEL(ai->topLabel), "Empty");
		gtk_label_set_text(GTK_LABEL(ai->bottomLabel), "Empty");
		return(FALSE);
	}

	/* Read the values and update the GUI */

	/* Advance the list pointer to the proper position */
        for(pos=0;pos<ai->displayPos;pos++) {
                list = g_list_next(list);
        }

	/* Get 1st Value */
	temp = (struct sensor *) list->data;
	sprintf(string, "%s: %.2f", temp->displayName, temp->value);

	gtk_label_set_text(GTK_LABEL(ai->topLabel), string);
	list = g_list_next(list);

	/* If there is a second value set the label */
	if(list != NULL) {
		/* Get 2nd Value */
		temp = (struct sensor *) list->data;
	        sprintf(string, "%s: %.2f", temp->displayName, temp->value);

	        gtk_label_set_text(GTK_LABEL(ai->bottomLabel), string);
		/*list = g_list_next(list);*/

		if(g_list_next(list) == NULL) {
			ai->displayPos = 0;
		} else {
			ai->displayPos = pos + 2;
		}

		list = g_list_next(list);
	} else {
		gtk_label_set_text(GTK_LABEL(ai->bottomLabel), "");
		ai->displayPos = 0;
	}

	/* Free the buffer */
	g_free(string);

	return(TRUE); /* Return TRUE to continue the timeout function */
}

gint updateSensors(gpointer data) {
	/* Update all of the sensor values */
	struct sensor *temp;
	struct appletInfo *ai;
	GList *list;

	ai = (struct appletInfo *) data;

	/* If there list is empty return FALSE to stop
	calling this function */
	if(ai->sensorList == NULL) {
		return(FALSE);
	}

	list = ai->sensorList;

	while(list != NULL) {
		temp = (struct sensor *) list->data;
		getFeature(&temp->nameStruct, temp->sensorNum, &temp->value);

		list = list->next;
	}

	return(TRUE); /* Return TRUE to continue the timeout function */
}

void aboutWindowCB(AppletWidget *caller, gpointer data) {
	const gchar *authors[] = {
		"Dan Siemon",
		NULL
	};

	gtk_widget_show (gnome_about_new ("sensor_sweep_applet", VERSION,
	"Copyright 1999.",
	(const gchar **) authors,
	_("sensor_sweep_applet is a small Gnome panel applet that monitors your systems health via the lm_sensors kernel modules."),
	NULL));
}

void destroy(GtkWidget *widget, gpointer data) {
	gtk_main_quit();
}
